#!/usr/bin/env python3
import os
import json
import pathlib
import argparse

from lottie_specs.schema import SchemaPath, Schema
from lottie_specs import type_info


def join_parts(
    json_data: dict,
    path: pathlib.Path,
    exclude: pathlib.Path
):
    defs = {}
    for subdir in sorted(path.iterdir()):
        dir_schema = {}
        if subdir.is_dir():
            for file_item in subdir.iterdir():
                if file_item.is_file() and file_item.suffix == ".json" and file_item != exclude:
                    with open(file_item, "r") as file:
                        try:
                            file_schema = json.load(file)
                        except Exception:
                            print(file_item)
                            raise
                    file_schema.pop("$schema", None)
                    dir_schema[file_item.stem] = file_schema
            defs[subdir.name] = dir_schema

    json_data["$defs"] = defs

    return json_data


def add_vals_to_unknown_object(
    objects,
    unknown_type_dict: dict
):
    types = []

    for ele in objects.concrete:
        type = ele.properties.get('ty', None)
        if type is not None and type.const is not None:
            types.append(type.const)

    unknown_type_dict["properties"]["ty"]["not"]["enum"] = types


root = pathlib.Path()

parser = argparse.ArgumentParser(description="Joins JSON schema in a single file")
parser.add_argument(
    "--input", "-i",
    type=pathlib.Path,
    default=root / "schema",
    help="Path to the input schema files"
)
parser.add_argument(
    "--root", "-r",
    type=pathlib.Path,
    default="root.json",
    help="Root schema file"
)
parser.add_argument(
    "--output", "-o",
    type=pathlib.Path,
    default=root / "docs" / "lottie.schema.json",
    help="Output file name"
)

args = parser.parse_args()
input_dir: pathlib.Path = args.input.resolve()
output_path: pathlib.Path = args.output
root_path: pathlib.Path = (input_dir / args.root).resolve()

with open(root_path) as file:
    json_data = json.load(file)

join_parts(json_data, input_dir, root_path)

schema = Schema(json_data)
ts = type_info.TypeSystem(schema)

add_vals_to_unknown_object(
    ts.from_path(SchemaPath("#/$defs/layers/all-layers")),
    json_data["$defs"]["layers"]["unknown-layer"]
)
add_vals_to_unknown_object(
    ts.from_path(SchemaPath("#/$defs/shapes/all-graphic-elements")),
    json_data["$defs"]["shapes"]["unknown-shape"]
)

os.makedirs(output_path.parent, exist_ok=True)

with open(output_path, "w") as file:
    json.dump(json_data, file, indent=4)



import json


class SchemaPath:
    """
    Path inside a Schema object
    """

    def __init__(self, path=None):
        if isinstance(path, str):
            self.chunks = [int(chunk) if chunk.isdigit() else chunk for chunk in path.strip("#/").split("/")]
        elif path is None:
            self.chunks = []
        elif isinstance(path, SchemaPath):
            self.chunks = list(path.chunks)
        else:
            self.chunks = list(path)

    def __itruediv__(self, chunk):
        self.chunks.append(chunk)

    def __truediv__(self, chunk):
        return SchemaPath(self.chunks + [chunk])

    def walk(self, schema):
        for chunk in self.chunks:
            if not self.valid_step(schema, chunk):
                return None
            schema = self.step(schema, chunk)
        return schema

    @classmethod
    def step(cls, schema, chunk):
        return schema[chunk]

    @classmethod
    def valid_step(cls, schema, chunk):
        if isinstance(chunk, int) and isinstance(schema, list):
            return 0 <= chunk < len(schema)
        elif chunk not in schema:
            return False
        return True

    def ensure_defs(self):
        if self.chunks[0] != "$defs":
            self.chunks.insert(0, "$defs")

    def __str__(self):
        return "#/" + "/".join(map(str, self.chunks))

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, repr(str(self)))


class Schema:
    """
    Class to access data from a JSON schema
    """

    def __init__(self, schema, path=None):
        self.schema = schema
        self.path = SchemaPath(path)

    def __getitem__(self, key):
        return self.child(key).value

    def __truediv__(self, key):
        return self.child(key)

    def __contains__(self, item):
        return isinstance(self.schema, dict) and item in self.schema

    def get(self, key, default=None):
        return self.schema.get(key, default) if isinstance(self.schema, dict) else default

    @property
    def value(self):
        return self.schema

    def __iter__(self):
        if isinstance(self.schema, list):
            iter = enumerate(self.schema)
        elif isinstance(self.schema, dict):
            iter = self.schema.items()
        else:
            return

        for key, value in iter:
            if isinstance(value, (object, list)):
                yield self / key

    def get_ref(self, path):
        path = SchemaPath(path)
        obj = path.walk(self)
        if obj is None:
            raise Exception("Schema object %s not found" % path)
        return Schema(obj, path)

    def items(self):
        if isinstance(self.schema, dict):
            for k, v in self.schema.items():
                yield k, Schema(v, self.path / k)
        return None

    def child(self, key):
        return Schema(SchemaPath.step(self.schema, key), self.path / key)

    @classmethod
    def load(cls, file):
        if hasattr(file, "read"):
            return cls(json.load(file))

        with open(file, "r") as f:
            return cls(json.load(f))



import xml.etree.ElementTree as etree

from markdown.treeprocessors import Treeprocessor
from markdown.inlinepatterns import InlineProcessor
from markdown.extensions import Extension
from markdown.extensions import toc

from .lottie_markdown import get_url


class TocDefListTreeProcessor(Treeprocessor):
    old_nest = toc.nest_toc_tokens

    def __init__(self, md):
        super().__init__(md)
        self.extra_toc = []
        # Patch toc.nest_toc_tokens to show glossary terms in the
        # table of contents (Hack)
        toc.nest_toc_tokens = self.patched_nest_toc_tokens

    def patched_nest_toc_tokens(self, toc_list):
        tl = TocDefListTreeProcessor.old_nest(toc_list + self.extra_toc)
        self.extra_toc = []
        return tl

    def run(self, root):
        self.extra_toc = []

        term: etree.Element
        for term in root.findall(".//dt"):
            if "id" not in term.attrib:
                text = toc.unescape(toc.strip_tags(toc.render_inner_html(term, self.md)))
                id = toc.slugify(text, "-")
                term.attrib["id"] = id
                link = etree.Element("a")
                for child in term:
                    term.remove(child)
                    link.append(child)
                link.text = term.text
                term.text = ""
                term.append(link)
                link.attrib["href"] = "#" + id
                self.extra_toc.append({
                    "level": 2,
                    "id": id,
                    "name": text
                })


class GlossaryLink(InlineProcessor):
    def __init__(self, md):
        super().__init__(r'\[\[([^\]\|]+)(?:\|([^\]]+))?\]\]', md)

    def handleMatch(self, match, data):
        link = etree.Element("a")
        title = match.group(1)
        id = toc.slugify(title, "-")
        link.attrib["href"] = get_url(self.md, "specs/glossary.md", id)
        link.text = match.group(2) or title
        return link, match.start(0), match.end(0)


class TocDefListExtension(Extension):
    def extendMarkdown(self, md):
        md.registerExtension(self)
        md.treeprocessors.register(TocDefListTreeProcessor(md), "toc_deflist", 175)
        md.inlinePatterns.register(GlossaryLink(md), "glossary_link", 175)


def makeExtension(**kwargs):
    return TocDefListExtension(**kwargs)



#!/usr/bin/env python3
import sys
import math
import inspect
import argparse
import lottie
from code_processing.loader import code_to_samples, SourceCode


class Bezier(lottie.objects.bezier.BezierView):
    def __init__(self):
        super().__init__(lottie.objects.bezier.Bezier(), False)

    def add_vertex(self, p: lottie.NVector):
        self.append(p)

    def set_in_tangent(self, p: lottie.NVector):
        self[-1].in_tangent = p

    def set_out_tangent(self, p: lottie.NVector):
        self[-1].out_tangent = p

    @property
    def closed(self):
        return self.bezier.closed

    @closed.setter
    def closed(self, v):
        self.bezier.closed = v


exec_globals = {
    "Color": lottie.Color,
    "Vector2D": lottie.NVector,
    "ELLIPSE_CONSTANT": 0.5519150244935105707435627,
    "Bezier": Bezier,
    "math": math,
}

default_args = {
    "NVector": lottie.NVector(200, 200),
    "float": 50,
    "int": 5,
}


def render_shape(func, args):
    anim = lottie.objects.Animation()
    lay = lottie.objects.layers.ShapeLayer()
    anim.add_layer(lay)
    lay.in_point = anim.in_point
    lay.out_point = anim.out_point

    shape = Bezier()
    func(shape, *args)
    lay.shapes.append(lottie.objects.shapes.Path(shape.bezier))
    lay.shapes.append(lottie.objects.shapes.Stroke(lottie.Color(1, 0.5, 0), 6))
    lay.shapes.append(lottie.objects.shapes.Fill(lottie.Color(1, 1, 0)))
    return anim


def main(argv):
    if argv.input:
        with open(argv.input) as f:
            code = f.read()
    else:
        print("Type code (end with ^D)")
        code = sys.stdin.read()

    if argv.view_code:
        data = code_to_samples(code)
        print(data[argv.view_code])
        parsed = data["ast"]
    else:
        parsed = SourceCode(code).ast

    local = {}
    exec(compile(parsed, "", "exec"), exec_globals, local)

    default_func = next(iter(local.keys()))

    if argv.func is not None:
        func_name = argv.func
    else:
        sys.stdout.write("Function name [%s]: " % default_func)
        sys.stdout.flush()
        func_name = sys.stdin.readline().strip()

    func = local[func_name or default_func]

    arg_spec = inspect.getfullargspec(func)

    given_args = {}

    if argv.args:
        for i in range(0, len(argv.args), 2):
            given_args[argv.args[i]] = argv.args[i + 1]

    args = []
    for i, arg in enumerate(arg_spec.args):
        if i == 0 and arg == "shape":
            continue
        annot = arg_spec.annotations[arg]
        typename = annot.__name__
        value = default_args[typename]
        if arg in given_args:
            value_raw = given_args[arg]
        else:
            sys.stdout.write("%s (%s) [%s]: " % (arg, typename, value))
            sys.stdout.flush()
            value_raw = sys.stdin.readline().strip()
        if value_raw:
            value = annot(*eval("[%s]" % value_raw))
        args.append(value)

    anim = render_shape(func, args)

    lottie.exporters.core.export_embedded_html(anim, "/tmp/out.html")


parser = argparse.ArgumentParser()
parser.add_argument("--input", "-i", help="File for the code input")
parser.add_argument("--func", "-f", default=None, help="Function name")
parser.add_argument("--args", nargs="+", help="Argument values for the function")
parser.add_argument("--view-code", "-c", metavar="lang", help="Display rendered code")

if __name__ == "__main__":
    args = parser.parse_args()
    main(args)



import pathlib
from setuptools import setup, find_namespace_packages

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "Readme.md").read_text(encoding="utf-8")

setup(
    name="lottie_specs",
    # version="1.0.1",
    version="0.1", # Dev version, until we release properly, after that sync with lottie-specs versioning
    description="Tools to load the Lottie standard specs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lottie/lottie-spec-python",
    author="Lottie Animation Community",
    packages=find_namespace_packages(where="src"),
    package_data={"lottie_specs.data": ["*.json"]},
    package_dir={"": "src"},
    python_requires=">=3.10, <4",
    scripts=[
        "bin/render_shape.py",
        "bin/schema-merge.py",
        "bin/schema-validate.py",
    ],
    extras_require={
        "docs": [
            "lxml==4.9.3",
            "source_translator==1.0.0",
            "graphviz==0.20.1",
            "mkdocs==1.5.3",
            "latex2mathml==3.77.0"
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/lottie/lottie-spec-python/issues",
        "Source": "https://github.com/lottie/lottie-spec-python",
    },
    # entry_points={
    #     "mkdocs.plugins": []
    # }
)



import json
from importlib.resources import files


def schema_path():
    """
    Returns the path to the schema file
    """
    return files("lottie_specs.data").joinpath("lottie.schema.json")


def load_specs(*a, **kw):
    """
    Loads the schema file into python objects
    """
    with open(schema_path(*a, **kw)) as fp:
        return json.load(fp)



import latex2mathml.converter

from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor
from markdown.blockprocessors import BlockProcessor


class InlineLatex(InlineProcessor):
    def __init__(self, md):
        super().__init__(r'\$([^\$\n]+)\$', md)

    def handleMatch(self, match, data):
        element = latex2mathml.converter.convert_to_element(match.group(1))
        return element, match.start(0), match.end(0)


class BlockLatex(BlockProcessor):
    fence = "$$"

    def __init__(self, md):
        super().__init__(md.parser)

    def test(self, parent, block):
        return block.startswith("$$")

    def run(self, parent, blocks):
        code = ""
        last_block = blocks.pop(0)[2:]
        while True:
            if last_block.endswith("$$"):
                code += last_block[:-2]
                break
            else:
                code += last_block
                last_block = blocks.pop(0)

        for chunk in code.split("$$\n$$"):
            element = latex2mathml.converter.convert_to_element(chunk, display="block")
            parent.append(element)

        return True


class LatexExtension(Extension):
    def extendMarkdown(self, md):
        md.inlinePatterns.register(InlineLatex(md), "inline_latex", 175)
        md.parser.blockprocessors.register(BlockLatex(md), "block_latex", 175)


def makeExtension(**kwargs):
    return LatexExtension(**kwargs)



from source_translator import AstTranslator, IndentationManager
from source_translator.naming import snake_to_lower_camel


class PseudoCode(AstTranslator):
    ops = {
        "Eq": "=",
        "NotEq": r"\neq",
        "Lt": "<",
        "LtE": r"\leq",
        "Gt": ">",
        "GtE": r"\geq",
        "Is": "=",
        "IsNot": r"\neq",
        "In": r"\in",
        "NotIn": r"\notin",
        "Add": "+",
        "Sub": "-",
        "Mult": r"\cdot",
        "MatMult": r"\times",
        "Div": r"\frac",
        "Mod": "%",
        "LShift": "<<",
        "RShift": ">>",
        "BitOr": "|",
        "BitXor": "^",
        "BitAnd": "&",
        "FloorDiv": "//",
        "Pow": "^",
        "Invert": "~",
        "Not": "\\neg",
        "UAdd": "+",
        "USub": "-",
        "And": "\\land",
        "Or": "\\lor",
    }

    def snake_sentence(self, name, upper):
        sentence = name.replace("_", " ")
        if upper:
            return sentence[0].upper() + sentence[1:]
        return sentence

    def expr_attribute(self, object, member):
        if object == "shape":
            if member == "closed":
                return "shape closed"
            return self.snake_sentence(member, True)
        if object == "math":
            return "\\" + member
        return "%s.%s" % (object, member)

    def begin_if(self, expr):
        self.push_code("If $%s$" % expr)

    def end_block(self):
        pass

    def begin_else(self):
        self.push_code("Otherwise")

    def declare(self, target, type, value, ast_value):
        self.push_code("$%s \\coloneq %s$" % (self.decorate_name(target), value))

    def format_comment(self, value):
        if len(value) == 0:
            self.push_code("")
        for line in value:
            self.push_code(line)

    def expression_statement(self, expr):
        if " " in expr or "$" in expr:
            self.push_code(expr)
        else:
            self.push_code("$%s$" % expr)

    def decorate_name(self, name):
        type = self.get_var_type(name)
        if type.startswith("\\mathbb{R}^"):
            return "\\vec{%s}" % name
        return name

    def expr_func(self, name, args):
        if name == "int":
            return args[0]

        if name == "round":
            return r"\lfloor %s \rceil" % ", ".join(args)

        if name == "floor":
            return r"\lfloor %s \rfloor" % ", ".join(args)

        if name == "ceil":
            return r"\lceil %s \rceil" % ", ".join(args)

        if name == "range":
            start = "0"
            end = "0"
            if len(args) == 1:
                end = args[0]
            elif len(args) == 2:
                start, end = args
            else:
                raise NotImplementedError

            return "[%s, %s)" % (start, end)

        is_sentence = " " in name
        if name == "Vector2D":
            name = ""
        code = name
        if is_sentence:
            code += " $"
        else:
            code += r"\left("
        code += ", ".join(args)
        if is_sentence:
            code += "$"
        else:
            code += r"\right)"
        return code

    def convert_name(self, name, annotation):
        if annotation:
            if name == "float":
                return "\\mathbb{R}"
            elif name == "int":
                return "\\mathbb{Z}"
            elif name == "Vector2D":
                return "\\mathbb{R}^2"

        if name in ("min", "max"):
            return "\\" + name
        if name == "ELLIPSE_CONSTANT":
            return "E_t"
        if name in ("alpha", "beta", "theta"):
            return "\\" + name

        name = name.strip("_")
        chunks = name.rsplit("_", 1)
        name = snake_to_lower_camel(chunks[0])
        if len(chunks) == 2:
            name += "_{%s}" % chunks[1]
        return self.decorate_name(name)

    def convert_constant(self, value, annotation):
        if value is None:
            return "nil"
        if isinstance(value, bool):
            return str(value).lower()
        return repr(value)

    def expr_binop(self, op, *operands):
        if op == "\\frac":
            return "%s{%s}{%s}" % (op, operands[0], operands[1])
        return (" %s " % op).join(operands)

    def assign(self, targets, value):
        for target in targets:
            if " " in target:
                if value == "true":
                    self.push_code("Set %s" % (target))
                elif value == "false":

                    self.push_code("Unset %s" % (target))
                else:
                    self.push_code("Set %s to %s" % (target, value))
            else:
                self.push_code("$%s \\coloneq %s$" % (target, value))

    def function_def(self, name, args, returns, body, is_async, is_method, is_getter):
        self.push_code(self.snake_sentence(name, True))

        with IndentationManager(self, False):
            args_start = 0
            if is_method and len(args.args) > 0 and args.args[0].arg in ("self", "cls"):
                args_start = 1

            if len(args.args) > args_start and args.args[args_start].arg == "shape":
                args_start += 1

            if len(args.args) > args_start:
                self.push_code("Inputs:")

                with IndentationManager(self, False):
                    for i in range(args_start, len(args.args)):
                        name = self.convert_name(args.args[i].arg, False)
                        if args.args[i].annotation:
                            type = self.expression_to_string(args.args[i].annotation, True)
                        else:
                            type = None
                        self.var_add(name, type)
                        arg = "$" + self.decorate_name(name)
                        if type:
                            arg += " \\in " + type

                        reverse_i = len(args.args) - i
                        if reverse_i <= len(args.defaults):
                            arg += " = %s" % self.expression_to_string(args.defaults[-reverse_i])
                        arg += "$"
                        self.push_code(arg)
                self.push_code("")

        with IndentationManager(self, False):
            self.convert_ast(body)

    def expr_compare(self, value, annotation):
        expr = self.expression_to_string(value.left, annotation)
        for cmp, op in zip(value.comparators, value.ops):
            expr += " " + self.expression_to_string(op, annotation)
            expr += " " + self.expression_to_string(cmp, annotation)
        return expr

    def begin_for(self, target, iter, is_async):
        code_start = "For each $%s$ in $%s$" % (target, iter)
        self.push_code(code_start)

    def convert_line_comment(self, comment):
        return comment or ""
